/*
 *  USBSink - aboutdialog.cc
 *
 *  Copyright (C) 2006-2007 Marko Anastasov
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "aboutdialog.h"
#include "globalstrings.h"

#include <config.h>

#include <list>

#include <glib/gi18n.h>

#include <gtkmm/aboutdialog.h>
#include <gtkmm/icontheme.h>

namespace USBSink
{

using std::list;

std::auto_ptr<Gtk::AboutDialog>
AboutDialog::create(Gtk::Window& parent)
{
  std::auto_ptr<Gtk::AboutDialog> dialog;
  dialog.reset(new Gtk::AboutDialog());

  dialog->set_name(PACKAGE_NAME);
  dialog->set_version(PACKAGE_VERSION);
  dialog->set_comments(_("A GNOME program for automatic file synchronization over USB"));
  
  list<Glib::ustring> authors;
  authors.push_back("Marko Anastasov <marko@marko.anastasov.name>");
  dialog->set_authors(authors);

  dialog->set_website("http://usbsink.sourceforge.net");
  dialog->set_website_label("Project Website");

  Glib::ustring license =
    "This program is free software; you can redistribute it and/or modify\n"
    "it under the terms of the GNU General Public License as published by\n"
    "the Free Software Foundation; either version 2 of the License, or\n"
    "(at your option) any later version.\n\n"
    
    "This program is distributed in the hope that it will be useful,\n"
    "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
    "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
    "GNU General Public License for more details.\n\n"
    
    "You should have received a copy of the GNU General Public License\n"
    "along with this program; if not, write to the \n"
    "Free Software Foundation, Inc., 59 Temple Place, Suite 330, \n"
    "Boston, MA  02111-1307  USA\n";
  dialog->set_license(license);

  // Translators: change this to your name, or separate multiple names with \n
  Glib::ustring translation_credits = _("translator-credits");
  dialog->set_translator_credits(translation_credits);
  
  const Glib::ustring icon_name = "usbsink";
  Glib::RefPtr<Gtk::IconTheme> theme = Gtk::IconTheme::get_default();

  if (theme->has_icon(icon_name))
  {
    try
    {
      Glib::RefPtr<Gdk::Pixbuf> icon =
        theme->load_icon(icon_name, 128, Gtk::ICON_LOOKUP_USE_BUILTIN);

      dialog->set_logo(icon);
    }
    catch (const Glib::Error& ex)
    {
      Glib::ustring message =
        "Couldn't load USBSink logo image from icon theme. " + ex.what();

      g_warning(message.c_str());
    }
  }

  dialog->set_transient_for(parent);
  dialog->signal_response().connect(
    sigc::hide(sigc::mem_fun(*dialog, &Gtk::Widget::hide)));

  return dialog;
}

} // namespace USBSink
