/*
 *  USBSink - addfilesdialog.h
 *
 *  Copyright (C) 2006 Marko Anastasov
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _ADDFILESDIALOG_H_
#define _ADDFILESDIALOG_H_

#include <sys/types.h>

#include <memory>
#include <list>

#include <gtkmm.h>

#include <sigc++/sigc++.h>

namespace Gnome { namespace Vfs { class Uri; } }

namespace USBSink
{

class AddFilesDialog : public sigc::trackable
{
 public:

  // from_removable: restrict selection to files on a removable drive,
  // ie this is a "removable to local" kind of task
  // removable_mount_point: mount point of the removable drive
  //
  explicit AddFilesDialog(Gtk::Window& parent,
			  bool from_removable = false,
			  const Glib::ustring& removable_mount_point = "");
  virtual ~AddFilesDialog();

  Gtk::Dialog* get_dialog() { return dialog_.get(); }

  // The signal which will be emitted when RESPONSE_OK occurs.
  // This way the dialog returns selected files.
  sigc::signal< void, std::list< Glib::RefPtr<Gnome::Vfs::Uri> >& >
    signal_file_uris_selected;

 protected:

  class FileListModelColumns : public Gtk::TreeModel::ColumnRecord
  {
  public:

    FileListModelColumns()
    {
      add(col_name);
      add(col_modified);
      add(col_mime_type);
      add(col_full_path);
    }

    Gtk::TreeModelColumn<Glib::ustring> col_name;
    Gtk::TreeModelColumn<Glib::ustring> col_modified;
    Gtk::TreeModelColumn<Glib::ustring> col_mime_type;
    Gtk::TreeModelColumn<Glib::ustring> col_full_path;
  };

  // signal handlers
  //
  virtual void on_toolbutton_add();
  virtual void on_toolbutton_remove();
  virtual void on_toolbutton_clear();

  virtual void on_treeview_drag_data_received(
    const Glib::RefPtr<Gdk::DragContext>& context,
    int x, int y,
    const Gtk::SelectionData& selection_data,
    guint info, guint time);

  virtual void on_response(int response_id);

  // member functions
  //
  virtual void load_xml();
  virtual void connect_signals();
  virtual void setup_filechooser(bool from_removable,
				 const Glib::ustring& removable_mount_point);
  virtual void setup_file_list_layout();

  virtual bool file_is_in_list(const Glib::ustring& uri);

  //virtual void add_uri_to_treeview(const Glib::ustring& uri);
  virtual void add_uri_to_treeview(const Glib::RefPtr< Gnome::Vfs::Uri >& uri);

  // dialog ptr
  //
  std::auto_ptr<Gtk::Dialog> dialog_;

  // widgets
  //
  Gtk::FileChooser*                filechooser_;

  Gtk::VPaned*                     vpaned_;

  Gtk::Toolbar*                    toolbar_;
  Gtk::ToolButton*                 toolbutton_add_;
  Gtk::ToolButton*                 toolbutton_remove_;
  Gtk::ToolButton*                 toolbutton_clear_;

  Gtk::TreeView*                   treeview_file_list_;
  FileListModelColumns             columns_files_;
  Glib::RefPtr<Gtk::ListStore>     treemodel_files_;
  Glib::RefPtr<Gtk::TreeSelection> file_list_selection_;

  // drag-and-drop targets
  //
  typedef std::list<Gtk::TargetEntry> list_dnd_targets;
  list_dnd_targets dnd_targets_;
};

} // namespace USBSink

#endif // _ADDFILESDIALOG_H_
