// -*- c++ -*-
/*
 *  USBSink - filelist.h
 *
 *  Copyright (C) 2006 Marko Anastasov
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _TASK_FILE_LIST_H_
#define _TASK_FILE_LIST_H_

#include "taskitem.h"

#include <tr1/memory>
#include <vector>

namespace USBSink
{

namespace TaskHandling
{

class FileList;

typedef std::tr1::shared_ptr<Item>     SharedItem;
typedef std::tr1::shared_ptr<FileList> SharedFileList;

class FileList : public Item
{
 public:

  virtual ~FileList() {}

  virtual Glib::ustring get_path() const = 0;

  virtual Glib::ustring get_type_string() const = 0;

  virtual std::vector<Glib::ustring> get_text_for_display() const = 0;

  virtual Glib::ustring              get_text_string_for_display() const = 0;

  virtual void save() const = 0;

  virtual bool filter(const Glib::ustring& str) const = 0;

  virtual void set_items(const std::list<SharedItem>& items) = 0;

  virtual bool contains_item(const Glib::ustring& name) const = 0;

  virtual void reset_item(SharedItem& item) = 0;

 protected:

  virtual void on_registry_request_response(SharedItem& item) = 0;
};

} // namespace TaskHandling

} // namespace USBSink

#endif // _TASK_FILE_LIST_H_
