// -*- c++ -*-
/*
 *  USBSink - filelisteditor.h
 *
 *  Copyright (C) 2006-2007 Marko Anastasov
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _FILE_LIST_EDITOR_H_
#define _FILE_LIST_EDITOR_H_

#include <map>

#include <gtkmm.h>

#include <sigc++/sigc++.h>

namespace USBSink
{

class FileListEditor : public sigc::trackable
{
public:

  explicit FileListEditor(Gtk::Window& parent);
  explicit FileListEditor(Gtk::Window& parent, const Glib::ustring& new_list);
  virtual ~FileListEditor();

  Gtk::Dialog* get_dialog() { return dialog_.get(); }

protected:

  class ListModelColumns : public Gtk::TreeModel::ColumnRecord
  {
  public:

    ListModelColumns() { add(col_name); }
    
    Gtk::TreeModelColumn<Glib::ustring> col_name;
  };

  void load_xml();
  void connect_signals();

  void setup_lists();

  bool list_name_in_use(const Glib::ustring& name) const;
  bool new_list(const Glib::ustring& new_list);

  bool on_treeview_select_function(const Glib::RefPtr<Gtk::TreeModel>& model,
                                   const Gtk::TreeModel::Path& path,
                                   bool path_currently_selected);

  void on_button_new_list_clicked();

  void on_button_test_clicked();
  void on_button_add_clicked();

  void on_combo_list_type_selection_changed();

  void on_button_cancel_clicked();
  void on_button_ok_clicked();

  // main dialog pointer
  std::auto_ptr<Gtk::Dialog> dialog_;

  // child widgets
  Gtk::TreeView*                   treeview_lists_;
  ListModelColumns                 columns_lists_;
  Glib::RefPtr<Gtk::ListStore>     treemodel_lists_;
  Glib::RefPtr<Gtk::TreeSelection> tree_selection_lists_;
  Gtk::Button*                     button_new_list_;

  Gtk::Entry*    entry_regexp_;
  Gtk::Entry*    entry_text_to_match_;
  Gtk::Button*   button_test_;
  Gtk::Label*    label_match_info_;
  Gtk::Button*   button_add_;

  Gtk::ComboBox* combobox_list_type_;
  Gtk::TextView* textview_list_content_;

  struct ListData
  {
    Glib::ustring type_string;
    Glib::RefPtr<Gtk::TextBuffer> buffer;
    bool type_changed;
    bool is_new;
  };

  std::map<Glib::ustring, ListData*> lists_;

  Gtk::Button* button_cancel_;
  Gtk::Button* button_ok_;

  // current list selection indicator
  Glib::ustring selected_list_name_;

  // last regexp evaluated
  Glib::ustring last_regexp_;

private:

  void init(Gtk::Window& parent);

};

} // namespace USBSink

#endif // _FILE_LIST_EDITOR_H_
