/*
 *  USBSink - flashdrivesdialog.h
 *
 *  Copyright (C) 2006 Marko Anastasov
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _FLASHDRIVESDIALOG_H_
#define _FLASHDRIVESDIALOG_H_

#include "halclient.h"

#include <memory>

namespace sigc
{
class trackable;
}

#include <gtkmm.h>

namespace USBSink
{

typedef Gtk::TreeModel::Children tree_children;

class FlashDrivesDialog : public sigc::trackable
{
 public:
	
  explicit FlashDrivesDialog(Gtk::Window& parent);
  virtual ~FlashDrivesDialog();

  Gtk::Dialog* get_dialog() { return dialog_.get(); }

 protected:

  // Columns in ListStore model of TreeView
  class ModelColumns : public Gtk::TreeModel::ColumnRecord
  {
  public:
    Gtk::TreeModelColumn<std::string>   col_udi;
    Gtk::TreeModelColumn<Glib::ustring> col_label;
    Gtk::TreeModelColumn<Glib::ustring> col_mnt_point;
    
    ModelColumns() { add(col_udi); add(col_label); add(col_mnt_point); }
  };
  
 private:
  
  std::auto_ptr<Gtk::Dialog> dialog_;
  
  const FlashDriveVector* drives_;
  
  ModelColumns list_columns_;
  
  Glib::RefPtr<Gtk::ListStore> listmodel_flash_drives_;
  
  Gtk::TreeView* treeview_flash_drives_;
  
  // object to manage the row selection
  Glib::RefPtr<Gtk::TreeSelection> list_selection_;
  
  Gtk::Button* button_mount_;
  Gtk::Button* button_unmount_;
  
  Gtk::Expander* expander_properties_;
  
  Gtk::Label* label_label_;
  Gtk::Label* label_free_space_;
  Gtk::Label* label_vendor_;
  Gtk::Label* label_model_;
  Gtk::Label* label_fs_;

  void load_xml();
  void connect_signals();
  void fill_listview();
  void update_expander(const SharedFlashDrive& sfd);

  virtual void on_response(int /* response_id */) { dialog_->hide(); }

  void on_drive_selection_changed();
  void on_mount_clicked();
  void on_unmount_clicked();
  void on_properties_expanded();

  void on_added(const SharedFlashDrive& sfd);
  void on_removed(const SharedFlashDrive& sfd);
  void on_mounted(const SharedFlashDrive& sfd);
  void on_unmounted(const SharedFlashDrive& sfd);

};

} // namespace USBSink

#endif
