/*
 *  USBSink - logviewdialog.h
 *
 *  Copyright (C) 2006 Marko Anastasov
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _LOGVIEW_DIALOG_H_
#define _LOGVIEW_DIALOG_H_

#include <gtkmm.h>

#include <sigc++/sigc++.h>

namespace USBSink
{

class LogViewDialog : public sigc::trackable
{
public:

  explicit LogViewDialog(Gtk::Window& parent);
  virtual ~LogViewDialog();

  Gtk::Dialog* get_dialog() { return dialog_.get(); }

protected:

  // tree model column class
  //
  class TaskListModelColumns : public Gtk::TreeModel::ColumnRecord
  {
  public:

    TaskListModelColumns() { add(col_id); add(col_name); }
    
    Gtk::TreeModelColumn<int> col_id; // not displayed
    Gtk::TreeModelColumn<Glib::ustring> col_name;
  };

  void load_xml();
  void connect_signals();

  void setup_task_list();

  bool on_treeview_select_function(const Glib::RefPtr<Gtk::TreeModel>& model,
				   const Gtk::TreeModel::Path& path,
				   bool path_currently_selected);
  void on_button_clear_clicked();
  void on_button_save_clicked();
  void on_button_ok_clicked();

  // main dialog pointer
  std::auto_ptr<Gtk::Dialog> dialog_;

  // child widgets
  Gtk::TreeView* treeview_task_list_;
  TaskListModelColumns columns_tasks_;
  Glib::RefPtr<Gtk::ListStore> treemodel_tasks_;
  Glib::RefPtr<Gtk::TreeSelection> task_list_selection_;

  Gtk::HPaned*   hpaned_;
  Gtk::TextView* textview_log_;

  Gtk::Button* button_clear_;
  Gtk::Button* button_save_;
  Gtk::Button* button_ok_;

  int selected_task_id_;

};

} // namespace USBSink

#endif // _LOGVIEW_DIALOG_H_
