/*
 *  USBSink - mainwindow.cc
 *
 *  Copyright (C) 2006-2007 Marko Anastasov
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "aboutdialog.h"
#include "edittaskdialog.h"
#include "flashdrivesdialog.h"
#include "globalstrings.h"
#include "filelisteditor.h"
#include "logviewdialog.h"
#include "mainwindow.h"
#include "newtaskdialog.h"
#include "task.h"
#include "taskmanager.h"
#include "utils.h"

#include <config.h>

#include <list>

#include <glib/gi18n.h>

#include <gtk/gtkmessagedialog.h>
#include <gtk/gtkwindow.h>

#include <libgnome/gnome-help.h>

using std::list;

namespace USBSink
{

MainWindow::MainWindow()
:
  menu_item_file_new_task_         (0),
  menu_item_file_quit_             (0),
  menu_item_edit_task_             (0),
  menu_item_edit_select_all_       (0),
  menu_item_edit_delete_task_      (0),
  menu_item_view_all_tasks_        (0),
  menu_item_view_running_tasks_    (0),
  menu_item_view_stopped_tasks_    (0),
  menu_item_view_available_drives_ (0),
  menu_item_view_file_lists_       (0),
  menu_item_view_transfer_log_     (0),
  menu_item_actions_start_         (0),
  menu_item_actions_stop_          (0),
  menu_item_help_contents_         (0),
  menu_item_help_about_            (0),
  toolbar_                         (0),
  new_button_                      (0),
  edit_button_                     (0),
  delete_button_                   (0),
  start_button_                    (0),
  stop_button_                     (0),
  task_view_                       (0),
  statusbar_                       (0)
{
  load_xml();
  
  gtk_window_set_default_icon_name("usbsink");
  
  window_->set_default_size(480, 300);

  menu_item_view_all_tasks_->set_active(true);
  menu_item_view_running_tasks_->set_active(false);
  menu_item_view_stopped_tasks_->set_active(false);

  setup_treeview();

  connect_signals();

  //TaskManager::init();
}

MainWindow::~MainWindow()
{
}

void
MainWindow::load_xml()
{
  using Gnome::Glade::Xml;

  try
  {
    const Glib::RefPtr<Xml> xml = Xml::create(glade_mainwindow_filename);
    
    Gtk::Window* dummy_window = 0;
    window_.reset(xml->get_widget("mainwindow", dummy_window));

    // menu items    
    xml->get_widget("new_task", menu_item_file_new_task_);
    xml->get_widget("quit", menu_item_file_quit_);
    xml->get_widget("edit_task", menu_item_edit_task_);
    xml->get_widget("select_all", menu_item_edit_select_all_);
    xml->get_widget("delete_task", menu_item_edit_delete_task_);
    xml->get_widget("all_tasks", menu_item_view_all_tasks_);
    xml->get_widget("running_tasks", menu_item_view_running_tasks_);
    xml->get_widget("stopped_tasks", menu_item_view_stopped_tasks_);
    xml->get_widget("available_flash_drives",menu_item_view_available_drives_);
    xml->get_widget("view_lists", menu_item_view_file_lists_);
    xml->get_widget("view_log", menu_item_view_transfer_log_);
    xml->get_widget("start", menu_item_actions_start_);
    xml->get_widget("stop", menu_item_actions_stop_);
    xml->get_widget("contents", menu_item_help_contents_);
    xml->get_widget("about", menu_item_help_about_);
    
    // toolbar
    xml->get_widget("toolbar", toolbar_);
		
    // toolbuttons
    xml->get_widget("new_button", new_button_);
    xml->get_widget("edit_button", edit_button_);
    xml->get_widget("delete_button", delete_button_);
    xml->get_widget("start_button", start_button_);
    xml->get_widget("stop_button", stop_button_);
    
    // treeview
    xml->get_widget("task_view", task_view_);
		
    // status bar
    xml->get_widget("statusbar", statusbar_);
  }
  catch (const Gnome::Glade::XmlError& xml_err)
  {
    g_error("%s", xml_err.what().c_str());
  }
}

void
MainWindow::connect_signals()
{
  using sigc::mem_fun;

  window_->signal_hide().connect(
    sigc::mem_fun(*(TaskHandling::Manager::instance()),
		  &TaskHandling::Manager::on_application_exit));

  // menu items
  menu_item_file_new_task_->signal_activate().connect(
    mem_fun(*this, &MainWindow::on_new_task));

  menu_item_file_quit_->signal_activate().connect(
    mem_fun(*this, &MainWindow::on_quit));

  menu_item_edit_task_->signal_activate().connect(
    mem_fun(*this, &MainWindow::on_edit_task));

  menu_item_edit_select_all_->signal_activate().connect(
    mem_fun(*this, &MainWindow::on_select_all));
  
  menu_item_edit_delete_task_->signal_activate().connect(
    mem_fun(*this, &MainWindow::on_delete_task));
  
  menu_item_view_all_tasks_->signal_toggled().connect(
    mem_fun(*this, &MainWindow::on_show_all_tasks));

  menu_item_view_running_tasks_->signal_toggled().connect(
    mem_fun(*this, &MainWindow::on_show_running_tasks));

  menu_item_view_stopped_tasks_->signal_toggled().connect(
    mem_fun(*this, &MainWindow::on_show_stopped_tasks));

  menu_item_view_available_drives_->signal_activate().connect(
    mem_fun(*this, &MainWindow::on_available_drives));

  menu_item_view_file_lists_->signal_activate().connect(
    mem_fun(*this, &MainWindow::on_view_file_list_editor));

   menu_item_view_transfer_log_->signal_activate().connect(
     mem_fun(*this, &MainWindow::on_view_log));

  menu_item_actions_start_->signal_activate().connect(
    mem_fun(*this, &MainWindow::on_start));
  
  menu_item_actions_stop_->signal_activate().connect(
    mem_fun(*this, &MainWindow::on_stop));

  menu_item_help_contents_->signal_activate().connect(
    mem_fun(*this, &MainWindow::on_help));

  menu_item_help_about_->signal_activate().connect(
    mem_fun(*this, &MainWindow::on_about));
  
  // tool buttons
  new_button_->signal_clicked().connect(
    mem_fun(*this, &MainWindow::on_new_task));
  
  edit_button_->signal_clicked().connect(
    mem_fun(*this, &MainWindow::on_edit_task));
  
  delete_button_->signal_clicked().connect(
    mem_fun(*this, &MainWindow::on_delete_task));
  
  start_button_->signal_clicked().connect(
    mem_fun(*this, &MainWindow::on_start));
  
  stop_button_->signal_clicked().connect(
    mem_fun(*this, &MainWindow::on_stop));

  // task manager
  TaskHandling::Manager* manager = TaskHandling::Manager::instance();

  manager->get_signal_task_added().connect(
    sigc::mem_fun(*this, &MainWindow::on_task_added));

  manager->get_signal_task_removed().connect(
    sigc::mem_fun(*this, &MainWindow::on_task_removed));

  manager->load_tasks();

  // connect to each task's signals
  list<SharedTask> tasks = manager->get_tasks();
  list<SharedTask>::iterator it(tasks.begin());
  list<SharedTask>::iterator end(tasks.end());

  for ( ; it != end; ++it)
  {
//     (*it)->signal_started.connect(
//       sigc::mem_fun(*this, &MainWindow::on_task_started));

//     (*it)->signal_stopped.connect(
//       sigc::mem_fun(*this, &MainWindow::on_task_stopped));

    (*it)->signal_begin_sync.connect(
      sigc::mem_fun(*this, &MainWindow::on_task_begin_sync));

    (*it)->signal_end_sync.connect(
      sigc::mem_fun(*this, &MainWindow::on_task_end_sync));
  }

  // halclient signals

  try
  {
    HalClient* hc = HalClient::instance();
    
    hc->signal_mounted.connect(
      sigc::mem_fun(*this, &MainWindow::on_drive_mounted));
    
    hc->signal_unmounted.connect(
      sigc::mem_fun(*this, &MainWindow::on_drive_unmounted));
  }
  catch (HalError& err)
  {
    g_warning("Hal exception: %s", err.what().c_str());
  }
}

void
MainWindow::setup_treeview()
{
  // create the tree model
  task_treemodel_ = Gtk::ListStore::create(task_columns_);
  task_view_->set_model(task_treemodel_);

  // set up the treeview selection object
  task_selection_ = task_view_->get_selection();
  task_selection_->set_mode(Gtk::SELECTION_MULTIPLE);

  // append column headers
  //
  // for Status, display a CellRendererPixbuf instead of a CellRendererText
  Gtk::CellRendererPixbuf cell;
  task_view_->append_column(_("Status"), cell);
  Gtk::TreeViewColumn* column = task_view_->get_column(0); // the status column
  column->add_attribute(cell.property_stock_id(), task_columns_.col_status);

  task_view_->append_column(_("Task name"), task_columns_.col_name);
  task_view_->append_column(_("Removable drive"), task_columns_.col_drive_label);
}

void
MainWindow::on_new_task()
{
  if (new_task_dialog_.get())
  {
     // The dialog is already open, and the user asked for it once again;
     // so move it where the user can see it.
     // Btw present() calls Widget::show() if a window is hidden.
     new_task_dialog_->get_dialog()->present();
  }
  else
  {
    std::auto_ptr<NewTaskDialog> dialog(new NewTaskDialog(*window_));
    
    dialog->get_dialog()->signal_hide().connect(
      sigc::mem_fun(*this, &MainWindow::on_new_task_hide));

    dialog->signal_file_list_editor_request.connect(
      sigc::mem_fun(*this, &MainWindow::on_file_list_editor));

    dialog->get_dialog()->show();
    
    new_task_dialog_ = dialog;
  }
}

void
MainWindow::on_new_task_hide()
{
  // Transfer ownership and let the destructor do the delete.
  const std::auto_ptr<NewTaskDialog> temp(new_task_dialog_);
}

/**
 * Handler for a request to open the file list editor.
 */
void
MainWindow::on_view_file_list_editor()
{
  if (file_list_editor_.get())
  {
     file_list_editor_->get_dialog()->present();
  }
  else
  {
    std::auto_ptr<FileListEditor> dialog(new FileListEditor(*window_));
    
    dialog->get_dialog()->signal_hide().connect(
      sigc::mem_fun(*this, &MainWindow::on_file_list_editor_hide));
    dialog->get_dialog()->show();
    
    file_list_editor_ = dialog;
  }
}

/**
 * Handler for a request to open the file list editor and to register a new
 * list under the specified name.
 */
void
MainWindow::on_file_list_editor(const Glib::ustring& list_name)
{
  if (file_list_editor_.get())
  {
     file_list_editor_->get_dialog()->present();
  }
  else
  {
    std::auto_ptr<FileListEditor> dialog;

    if (list_name != Glib::ustring())
    {
      dialog.reset(new FileListEditor(*window_, list_name));
    }
    else
    {
      dialog.reset(new FileListEditor(*window_));
    }
    
    dialog->get_dialog()->signal_hide().connect(
      sigc::mem_fun(*this, &MainWindow::on_file_list_editor_hide));
    dialog->get_dialog()->show();
    
    file_list_editor_ = dialog;
  }
}

void
MainWindow::on_file_list_editor_hide()
{
  const std::auto_ptr<FileListEditor> temp(file_list_editor_);
}

void
MainWindow::on_edit_task()
{
  list<Gtk::TreeModel::Path> rows(task_selection_->get_selected_rows());

  if (! rows.empty())
  {
    Gtk::TreeModel::Row row = *(task_treemodel_->get_iter(*rows.begin()));

    TaskHandling::Manager* tm = TaskHandling::Manager::instance();
    SharedTask sel_task = tm->get_task(row[task_columns_.col_id]);

    if (edit_task_dialog_.get())
    {
      edit_task_dialog_->get_dialog()->present();
    }
    else
    {
      std::auto_ptr<EditTaskDialog> dialog(
	new EditTaskDialog(*window_, sel_task));
      
      dialog->get_dialog()->signal_hide().connect(
	sigc::mem_fun(*this, &MainWindow::on_edit_task_hide));

      dialog->signal_file_list_editor_request.connect(
        sigc::mem_fun(*this, &MainWindow::on_file_list_editor));
      
      dialog->get_dialog()->show();

      edit_task_dialog_ = dialog;
    }
  }
}

void
MainWindow::on_edit_task_hide()
{
  const std::auto_ptr<EditTaskDialog> temp(edit_task_dialog_);
}


void
MainWindow::on_delete_task()
{
  list<Gtk::TreeModel::Path> rows(task_selection_->get_selected_rows());

  if (! rows.empty())
  {
    Gtk::TreeModel::Row row = *(task_treemodel_->get_iter(*rows.begin()));

    int task_id = row[task_columns_.col_id];

    TaskHandling::Manager* manager = TaskHandling::Manager::instance();
    const Glib::ustring task_name = manager->get_task_name(task_id);

    Gtk::MessageDialog dialog(
      *(get_window()),
      Util::i18n_printf(_("Are you sure you want to permanently delete '%s'?"),
                        task_name.c_str()),
      false /* use_markup */,
      Gtk::MESSAGE_WARNING,
      Gtk::BUTTONS_NONE);

    dialog.set_title(_("Delete task?"));

    Gtk::StockID stock_cancel(Gtk::Stock::CANCEL);
    Gtk::StockID stock_delete(Gtk::Stock::DELETE);

    dialog.add_button(stock_cancel, Gtk::RESPONSE_CANCEL);
    dialog.add_button(stock_delete, Gtk::RESPONSE_OK);

    dialog.set_default_response(Gtk::RESPONSE_OK);

    dialog.set_secondary_text(_("If you delete a task, its file and associated list will be permanently lost."));

    int result = dialog.run();

    if (result == Gtk::RESPONSE_OK)
      manager->remove_task(task_id);
  }
}

void
MainWindow::on_select_all()
{
  task_selection_->select_all();
}

void
MainWindow::on_start()
{
  list<Gtk::TreeModel::Path> rows(task_selection_->get_selected_rows());

  if (! rows.empty())
  {
    list<Gtk::TreeModel::Path>::iterator it(rows.begin());
    list<Gtk::TreeModel::Path>::iterator end(rows.end());

    for ( ; it != end; ++it)
    {
      Gtk::TreeModel::Row row = *(task_treemodel_->get_iter(*it));

      TaskHandling::Manager* tm = TaskHandling::Manager::instance();
      SharedTask sel_task = tm->get_task(row[task_columns_.col_id]);

      if (sel_task->get_drive().get() == 0)
      {
	Glib::ustring message(
	  Util::i18n_printf(_("Cannot start task %s"),
	                    sel_task->get_name().c_str()));
	Gtk::MessageDialog err_dialog(*(get_window()),
				      message, false, Gtk::MESSAGE_ERROR,
				      Gtk::BUTTONS_OK, true);
	Glib::ustring err_details(_("Target drive is not present."));
	err_dialog.set_secondary_text(err_details);
	err_dialog.run();
      }
      else
      {
	if (! sel_task->get_drive()->is_mounted())
	{
	  Glib::ustring message(
	    Util::i18n_printf(_("Cannot start task %s"),
	                    sel_task->get_name().c_str()));
	  Gtk::MessageDialog err_dialog(*(get_window()),
					message, false, Gtk::MESSAGE_ERROR,
					Gtk::BUTTONS_OK, true);
	  
	  // TODO: suggest and offer actions with some buttons!!
	  Glib::ustring err_details(_("Target drive is not mounted."));
	  err_dialog.set_secondary_text(err_details);
	  err_dialog.run();
	}
	else
	{
	  if (! sel_task->is_running())
	  {
	    sel_task->start();
	  }
	}
      }
    }
  }
}

void
MainWindow::on_stop()
{
  list<Gtk::TreeModel::Path> rows(task_selection_->get_selected_rows());

  if (! rows.empty())
  {
    list<Gtk::TreeModel::Path>::iterator it(rows.begin());
    list<Gtk::TreeModel::Path>::iterator end(rows.end());

    for ( ; it != end; ++it)
    {
      Gtk::TreeModel::Row row = *(task_treemodel_->get_iter(*it));
      
      TaskHandling::Manager* tm = TaskHandling::Manager::instance();
      SharedTask sel_task = tm->get_task(row[task_columns_.col_id]);
      
      if (sel_task->is_running())
	sel_task->stop("");
    }
  }
}

void
MainWindow::on_view_log()
{
  if (log_view_dialog_.get())
  {
    log_view_dialog_->get_dialog()->present();
  }
  else
  {
    std::auto_ptr<LogViewDialog> dialog(new LogViewDialog(*window_));
    
    dialog->get_dialog()->signal_hide().connect(
      sigc::mem_fun(*this, &MainWindow::on_log_view_dialog_hide));
    dialog->get_dialog()->show();
    
    log_view_dialog_ = dialog;
  }
}

void
MainWindow::on_log_view_dialog_hide()
{
  const std::auto_ptr<LogViewDialog> temp(log_view_dialog_);
}

void
MainWindow::on_help()
{
  GError* error = NULL;

  gnome_help_display ("usbsink", NULL, &error);
  g_debug("%s", PACKAGE_NAME);

  if (error)
  {
    GtkWidget *dialog;

    dialog = gtk_message_dialog_new (GTK_WINDOW (get_window()->gobj()),
                                     GTK_DIALOG_DESTROY_WITH_PARENT,
                                     GTK_MESSAGE_ERROR,
                                     GTK_BUTTONS_CLOSE,
                                     _("Could not display help for USBSink\n"
                                       "%s"),
                                     error->message);
    g_signal_connect_swapped (dialog, "response",
                              G_CALLBACK (gtk_widget_destroy),
                              dialog);
    gtk_widget_show (dialog);
    g_error_free (error);
  }
}

void
MainWindow::on_about()
{
  if (about_dialog_.get())
  {
    about_dialog_->present();
  }
  else
  {
    std::auto_ptr<Gtk::Dialog> dialog(AboutDialog::create(*window_));
    
    dialog->signal_hide().connect(
      sigc::mem_fun(*this, &MainWindow::on_about_dialog_hide));
    dialog->show();
    
    about_dialog_ = dialog;
  }
}

void
MainWindow::on_about_dialog_hide()
{
  const std::auto_ptr<Gtk::Dialog> temp(about_dialog_);
}

void
MainWindow::show_tasks(ShowTasks choice)
{
  task_treemodel_->clear();

  TaskHandling::Manager* manager = TaskHandling::Manager::instance();
  list<SharedTask> tasks = manager->get_tasks();
  list<SharedTask>::iterator it(tasks.begin());
  list<SharedTask>::iterator end(tasks.end());

  for ( ; it != end; ++it)
  {
    switch (choice)
    {
    case SHOW_TASKS_RUNNING:
      if ((*it)->is_running())
	display_task(*it);
      break;

    case SHOW_TASKS_STOPPED:
      if (! (*it)->is_running())
	display_task(*it);
      break;

    case SHOW_TASKS_ALL:
      display_task(*it);
      break;
    }
  }
}

void
MainWindow::on_show_all_tasks()
{
  if (menu_item_view_all_tasks_->get_active())
    show_tasks(SHOW_TASKS_ALL);
}

void
MainWindow::on_show_running_tasks()
{
  if (menu_item_view_running_tasks_->get_active())
    show_tasks(SHOW_TASKS_RUNNING);
}

void
MainWindow::on_show_stopped_tasks()
{
  if (menu_item_view_stopped_tasks_->get_active())
    show_tasks(SHOW_TASKS_STOPPED);
}

void
MainWindow::on_available_drives()
{
  if (flash_drives_dialog_.get())
  {
    flash_drives_dialog_->get_dialog()->present();
  }
  else
  {
    std::auto_ptr<FlashDrivesDialog> dialog(new FlashDrivesDialog(*window_));
    
    dialog->get_dialog()->signal_hide().connect(
      sigc::mem_fun(*this, &MainWindow::on_available_drives_hide));
    dialog->get_dialog()->show();
    
    flash_drives_dialog_ = dialog;
  }
}

void
MainWindow::on_available_drives_hide()
{
  const std::auto_ptr<FlashDrivesDialog> temp(flash_drives_dialog_);
}

void
MainWindow::on_quit()
{
  window_->hide();
}

/**
 * Caller function should always check if returned iterator is valid.
 */
tree_children::iterator
MainWindow::get_tree_iter_for_task(int task_id)
{
  tree_children children = task_treemodel_->children();
  tree_children::iterator iter = children.begin();
  tree_children::iterator end = children.end();
  
  for (; iter != end; ++iter)
  {
    if ((*iter)[task_columns_.col_id] == task_id)
    {
      break;
    }
  }

  return iter;
}

void
MainWindow::display_task(const SharedTask& task)
{
  // add a new row to the treeview
  Gtk::TreeModel::Row row = *(task_treemodel_->append());

  row[task_columns_.col_id] = task->get_id();

  if ((task->get_start_time_option() == TaskHandling::START_TIME_OPTION_NOW) &&
      task->is_running())
  {
    // col_status is a stockID string
    row[task_columns_.col_status] = Gtk::Stock::YES.id;
  }
  else
  {
    row[task_columns_.col_status] = Gtk::Stock::STOP.id;
  }

  row[task_columns_.col_name] = task->get_name();

  if (task->get_drive() != 0)
  {
    if (task->get_drive()->is_mounted())
    {
      row[task_columns_.col_drive_label] = task->get_drive()->get_label();
    }
    else
    {
      // the drive is present, but unmounted
      row[task_columns_.col_drive_label] =
	Util::i18n_printf(_("%s: not mounted"),
	                  task->get_drive()->get_label().c_str());
    }
  }
  else
  {
    // the drive is not present
    row[task_columns_.col_drive_label] = _("Not present");
  }
}

void
MainWindow::on_task_added(const SharedTask& task)
{
  display_task(task);

  // connect to its signals
  task->signal_started.connect(
    sigc::mem_fun(*this, &MainWindow::on_task_started));

  task->signal_stopped.connect(
    sigc::mem_fun(*this, &MainWindow::on_task_stopped));
}

void
MainWindow::on_task_removed(int task_id)
{
  tree_children::iterator iter(get_tree_iter_for_task(task_id));

  if (iter)
  {
    // erase the corresponding row
    task_treemodel_->erase(iter);
  }
}

void
MainWindow::on_task_started(TaskHandling::Task* task)
{
  tree_children::iterator iter(get_tree_iter_for_task(task->get_id()));

  if (iter)
  {
    Gtk::TreeModel::Row row = *iter;
 
    row[task_columns_.col_status] = Gtk::Stock::YES.id;
    row[task_columns_.col_drive_label] = task->get_drive()->get_label();
  }
}

void
MainWindow::on_task_stopped(TaskHandling::Task* task,
                            const Glib::ustring& note)
{
  tree_children::iterator iter(get_tree_iter_for_task(task->get_id()));

  if (iter)
  {
    Gtk::TreeModel::Row row = *iter;
 
    row[task_columns_.col_status] = Gtk::Stock::STOP.id;

    Glib::ustring label(row[task_columns_.col_drive_label]);
    if (note != Glib::ustring())
      label += " : " + note;

    row[task_columns_.col_drive_label] = label;
  }
}

void
MainWindow::on_task_begin_sync(TaskHandling::Task* task)
{
  statusbar_->push(Util::i18n_printf(_("Synchronizing files for %s"),
                                     task->get_name().c_str()));

  tree_children::iterator iter(get_tree_iter_for_task(task->get_id()));

  if (iter)
  {
    Gtk::TreeModel::Row row = *iter;
    row[task_columns_.col_status] = Gtk::Stock::EXECUTE.id;
  }
}

void
MainWindow::on_task_end_sync(TaskHandling::Task* task)
{
  statusbar_->push(Util::i18n_printf(_("Synchronized files for %s"),
                                     task->get_name().c_str()));

  tree_children::iterator iter(get_tree_iter_for_task(task->get_id()));

  if (iter)
  {
    Gtk::TreeModel::Row row = *iter;
    row[task_columns_.col_status] = Gtk::Stock::YES.id;
  }
}

void
MainWindow::on_drive_mounted(const SharedFlashDrive& flash)
{
  statusbar_->push(Util::i18n_printf(_("Drive %s mounted"),
                                     flash->get_label().c_str()));
}

void
MainWindow::on_drive_unmounted(const SharedFlashDrive& flash)
{
  statusbar_->push(Util::i18n_printf(_("Drive %s unmounted"),
                                     flash->get_label().c_str()));
}

} // namespace USBSink
