// -*- c++ -*-
/*
 *  USBSink - taskmanager.h
 *
 *  Copyright (C) 2006 Marko Anastasov
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef __TASK_MANAGER_H__
#define __TASK_MANAGER_H__

#include "uncopyable.h"

#include <list>
#include <memory>

#include <tr1/memory>

#include <glibmm/ustring.h>

namespace USBSink
{

namespace TaskHandling
{

class FileList;
class Task;

typedef std::tr1::shared_ptr<FileList> SharedFileList;
typedef std::tr1::shared_ptr<Task> SharedTask;

class Manager : private Uncopyable
{
 public:

  static Manager* instance();
  ~Manager();

  static void init();
  void        load_tasks();

  void add_task(SharedTask& task, bool save = true, bool start = true);
  int  get_new_task_id() const;
  void remove_task(int task_id);

  SharedTask                  get_task(const int id);
  SharedTask                  get_task(const Glib::ustring& file_name);
  const std::list<SharedTask> get_tasks() const { return tasks_; }
  const Glib::ustring         get_task_name(int id) const;

  bool task_exists(const Glib::ustring& name) const;

  sigc::signal< void, const SharedTask& >& get_signal_task_added();
  sigc::signal< void, int >&               get_signal_task_removed();

  void on_task_loaded(SharedTask& task, bool start);

  // HalClient signal handlers
  void on_drive_mounted(const SharedFlashDrive& drive);
  void on_drive_unmounted(const SharedFlashDrive& drive);

  void on_application_exit();

 protected:

  Manager();

  std::list<SharedTask> get_tasks_on_drive(const std::string& udi);

  void on_file_list_reset(SharedFileList& list);

  sigc::signal<void, const SharedTask&> signal_task_added_;
  sigc::signal<void, int>               signal_task_removed_;

 private:

  static std::auto_ptr<Manager> instance_;

  std::list<SharedTask> tasks_;
};

} // namespace TaskHandling

} // namespace USBSink

#endif // __TASK_MANAGER_H__
